@@ -20,10 +20,16 @@
     assert-equal
     assert-eof
     assert-starts-with
-    assert-matches)
+    assert-matches
+    assert-superblock-untouched
+    assert-member?
+    assert-raises-thunk
+    assert-raises
+    assert-every)
 
   (import
     (chezscheme)
+    (bcache block-manager)
     (fmt fmt)
     (list-utils)
     (logging)
@@ -31,6 +37,7 @@
     (regex)
     (temp-file)
     (utils)
+    (only (srfi s1 lists) every)
     (srfi s8 receive))
 
   ;;;--------------------------------------------------------------------
@@ -91,7 +98,8 @@
       (fmt #t (cat (fn keys) nl))
       (flush))
 
-    (for-each describe (cons '() (reverse (cdr (reverse keys)))) keys))
+    (unless (null? keys)
+        (for-each describe (cons '() (reverse (cdr (reverse keys)))) keys)))
 
   (define (describe-scenarios keys)
     (fmt-scenarios keys
@@ -180,19 +188,31 @@
 
   ;;-----------------------------------------------
 
-  ;; FIXME: don't hard code this
-  (define tools-version "0.7.2")
+  (define tools-version
+          (chomp
+            (with-input-from-file "../VERSION"
+                                  (lambda ()
+                                    (get-line (current-input-port))))))
 
-  (define (tool-name sym)
+  ;;-----------------------------------------------
+  ;; A 'tool' is a function that builds up a command line.  This can then be
+  ;; passed to the functions in the (process) library.
+  (define (tool-path sym)
     (define (to-underscore c)
       (if (eq? #\- c) #\_ c))
 
-    (list->string (map to-underscore (string->list (symbol->string sym)))))
+    (string-append "../bin/"
+                   (list->string
+                     (map to-underscore
+                          (string->list
+                            (symbol->string sym))))))
 
   (define-syntax define-tool
     (syntax-rules ()
-      ((_ tool-sym) (define (tool-sym . flags)
-                      (apply run-ok (tool-name 'tool-sym) flags)))))
+      ((_ sym) (define sym
+                 (let ((path (tool-path 'sym)))
+                  (lambda args
+                    (build-command-line (cons path args))))))))
 
   (define (assert-equal str1 str2)
     (unless (equal? str1 str2)
@@ -221,5 +241,45 @@
           (unless ((regex pattern) str)
                   (fail (fmt #f "string should match: " pattern ", " str))))
 
+  (define (all-zeroes? ptr count)
+    (let ((u8-ptr (make-ftype-pointer unsigned-8 ptr)))
+     (let loop ((i 0))
+      (if (= count i)
+          #t
+          (if (zero? (ftype-ref unsigned-8 () u8-ptr i))
+              (loop (+ i 1))
+              #f)))))
+
+  (define (assert-superblock-untouched md)
+    (with-bcache (cache md 1)
+      (with-block (b cache 0 (get-flags))
+        (unless (all-zeroes? (block-data b) 4096)
+                (fail "superblock contains non-zero data")))))
+
+  (define (assert-member? x xs)
+    (unless (member x xs)
+      (fail (fmt #f "expected " (wrt x) "to be a member of " (wrt xs)))))
+
+  (define (assert-raises-thunk thunk)
+    (call/cc
+      (lambda (k)
+        (with-exception-handler
+          (lambda (x)
+            (if (error? x)
+                (k #f)
+                (raise x)))
+          thunk)
+        (fail "expected an exception to be raised"))))
+
+  (define-syntax assert-raises
+    (syntax-rules ()
+      ((_ b1 b2 ...)
+       (assert-raises-thunk
+         (lambda ()
+           b1 b2 ...)))))
+
+  (define (assert-every pred . args)
+    (unless (apply every pred args)
+      (fail "assert-every failed")))
   )
 
